//----------------------------------------------//
//	Helpfull function for deciding mesh size	  //
//----------------------------------------------//

// a1: mesh size of the first cell 
// aN: mesh size of the last cell
Macro CalExpansionRatio
   NumGrid = Log(1 - Sn * (1-q) / a0) / Log(q);
   NumGrid = Round(NumGrid) + 1;
   aN	   = a0 * q^(NumGrid - 1);
   NumNode = NumGrid;   
Return

//--------------------------------------------------//
//  GMSH project created by Brage Møller-Pettersen  //
//--------------------------------------------------//

//--------------------------------------------------------------------//
// 	Original vessel used as a mockup model:				    		  //
// 	UT 5400 WP vessel range Windfarm Service Operation Vessel (SOV)	  //
//--------------------------------------------------------------------//
LoA					        = 82.0;	        // [m]					  //
L_pp				        = 72.1;	        // [m]					  //
Breadth				        = 18.0;         // [m]					  //
Draft				        = 5.0;          // [m]					  //
thrusterDiameter	        = 2.5;	        // [m]					  //
currentVelocity             = 1.0;          // [m/s]                  //
turbulenceIntensity         = 0.02;         // [%]                    //
scaleRatio			        = 1/20;         // [-]					  //
//--------------------------------------------------------------------//

//////////////////////////////////////////
//	Coefficients Deciding Mesh Shape	//
//////////////////////////////////////////
//+
// DoT = Depth/Draft
DoT = 3;
// Radii of curvature in hull corners
Radii = 1;	                        // [m]	// Must be smaller than draft
k = Radii * scaleRatio;

//////////////////////////////////////
//	Outer points of the hull	    //
//////////////////////////////////////
//+
yh = Breadth * 0.5 * scaleRatio;
zh = Draft * scaleRatio;

////////////////////
//	Waterdepth	//
////////////////////
//+
d = DoT*zh;		clearance = d - zh;

pDia = thrusterDiameter * scaleRatio;
outerCoeff = clearance * 0.9;

//////////////////////////////////////
//	z coordinates from the top	    //
//////////////////////////////////////
//+
z1 = zh - k;
zInn = zh + clearance*0.3;  zOut = zh + outerCoeff;

//////////////////////////////////////
//	y coordinates from inlet	    //
//////////////////////////////////////
//+
ySt = yh * 4;			    yEn = yh * 4.5;
yInn = yh + clearance*0.3;  yOut = yh + outerCoeff;
y1 = yh - k;

////////////////////////////////////////////
//	y & z coordinates for circular arcs   //
////////////////////////////////////////////
//+
Oy = y1;
Oz = z1;
rh = yh - Oy;
rInn = yInn - Oy;
rOut = yOut - Oy;

////////////////////////////////////////////////////////////////////
//	y & z coordinates for the diagonal line intersecting arcs	  //
////////////////////////////////////////////////////////////////////
//+
yd1 = Oy + rh * Sqrt(2) * 0.5;		zd1 = Oz + rh * Sqrt(2) * 0.5;
yd2 = Oy + rInn * Sqrt(2) * 0.5;	zd2 = Oz + rInn * Sqrt(2) * 0.5;
yd3 = Oy + rOut * Sqrt(2) * 0.5;	zd3 = Oz + rOut * Sqrt(2) * 0.5;
yd4 = Oy + (d-z1) * Tan(Pi/4);		zd4 = d;

////////////////////////////////////////////////////////////////////
//			The Length Of Each Meshing Line			//
////////////////////////////////////////////////////////////////////
//+
L_vec = {
    ySt - yd3,                        //  0
    yOut - yInn,                      //  1
    yInn - yh,                        //  2
    yEn - yOut,                       //  3
    z1,                               //  4
    Pi * 2 * k * 0.125,               //  5
    Pi * 2 * k * 0.125,               //  6
    2 * y1,                           //  7
    Pi * 2 * k * 0.125,               //  8
    Pi * 2 * k * 0.125,               //  9
    z1,                               // 10
    d - zd3                           // 11
};

//////////////////////////////////////////////
// Calculate Appropriate Number Of Elements //
//////////////////////////////////////////////
//+
//////////////////////////////////////////////
//      Indexes:                            //
// 0 = Surface to bottom inlet side         //
// 1 = Outer subdomain                      //
// 2 = Inner subdomain                      //
// 3 = Surface to bottom outlet side        //
// 4 = Inlet to top of hull                 //
// 5 = Inlet to corner of hull              //
// 6 = bottom to corner of hull inlet side  //
// 7 = bottom to hull                       //
// 8 = bottom to corner of hull outlet side //
// 9 = Outlet to corner of hull             //
// 10 = Outlet to top of hull               //
// 11 = Inlet to outlet                     //
//////////////////////////////////////////////

// Define the smallest cell side length:

meshSizes = {       // nCells:      id:
        6.04e-3,    //   10076       0
        3.4e-3,     //   24943       1
        2.15e-3,    //   50399       2
        1.64e-3,    //   75485       3
        1.36e-3,    //   99550       4
        1.17e-3,    //  124598       5
        1.03e-3,    //  149216       6
        9.23e-4,    //  174446       7
        8.41e-4,    //  200013       8
        7.74e-4,    //  225075       9
        7.17e-4,    //  250218      10
        4.32e-4,    //  500075      11
        3.19e-4,    //  749268      12
        2.56e-4     // 1001452      13
};

fine = meshSizes[4];
semiFine = fine * 1.25;
coarseFine = fine * 1.5;
coarse = fine * 2;
veryCoarse = fine * 12;

/*  Suggestion 
    fine = meshSizes[];
    semiFine = fine * 1.1;
    coarseFine = semiFine * 1.1;
    coarse = coarseFine * 1.1;
    veryCoarse = coarse * 1.1;
*/

cellMin = {
    veryCoarse,   //  0               // Matches largest cell in 1
    semiFine,     //  1               // Matches largest cell in 2
    semiFine,     //  2               // Matches largest cell in 3
    fine,         //  3
    semiFine,     //  4               // Matches largest cell in 1
    fine,         //  5
    fine,         //  6
    fine,         //  7
    fine,         //  8
    fine,         //  9
    fine,         // 10
    fine,         // 11
    fine,         // 12
    coarseFine,   // 13
    fine          // 14
};
// Define expansion ratios
r_exp = {
    1.1,          //  0
    1.0125,       //  1
    1.0125,       //  2
    1.1,          //  3
    1.025,        //  4
    1,            //  5
    1,            //  6
    1,            //  7
    1,            //  8
    1,            //  9
    1.025,        // 10
    1             // 11
};
// Preallocate vectors for largest grid cells and number of nodes
cellMax = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
nCells = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

// 2 = Inner subdomain
Sn = L_vec[2];
q = r_exp[2];
a0 = cellMin[2];
Call CalExpansionRatio;
cellMax[2] = aN;
nCells[2] = NumNode;

// 1 = Outer subdomain
Sn = L_vec[1];
q = r_exp[1];
a0 = cellMax[2];       cellMin[1] = cellMax[2];
Call CalExpansionRatio;
cellMax[1] = aN;
nCells[1] = NumNode;

// 0 = Surface to bottom inlet side
Sn = L_vec[0];
q = r_exp[0];
a0 = cellMax[1];       cellMin[0] = cellMax[1];
Call CalExpansionRatio;
cellMax[0] = aN;
nCells[0] = NumNode;

// 3 = Surface to bottom outlet side
Sn = L_vec[3];
q = r_exp[3];
a0 = cellMax[1];       cellMin[3] = cellMax[1];
Call CalExpansionRatio;
cellMax[3] = aN;
nCells[3] = NumNode;

// 4 = Inlet to top of hull
Sn = L_vec[4];
q = r_exp[4];
a0 = cellMin[4];
Call CalExpansionRatio;
cellMax[4] = aN;
nCells[4] = NumNode;

// 5 = Inlet to corner of hull
cellMax[5] =  cellMin[5];
nCells[5] = Round(L_vec[5]/cellMin[5]);

// 6 = bottom to corner of hull inlet side
cellMax[6] =  cellMin[6];
nCells[6] = Round(L_vec[6]/cellMin[6]);

// 7 = bottom to hull
cellMax[7] =  cellMin[7];
nCells[7] = Round(L_vec[7]/cellMin[7]);

// 8 = bottom to corner of hull outlet side
cellMax[8] =  cellMin[8];
nCells[8] = Round(L_vec[8]/cellMin[8]);

// 9 = Outlet to corner of hull
cellMax[9] =  cellMin[9];
nCells[9] = Round(L_vec[9]/cellMin[9]);

// 10 = Outlet to top of hull
Sn = L_vec[10];
q = r_exp[10];
a0 = cellMin[10];
Call CalExpansionRatio;
cellMax[10] = aN;
nCells[10] = NumNode;

// 11 = Inlet to outlet
cellMin[11] = cellMax[1];
cellMax[11] =  cellMin[11];
nCells[11] = Round(L_vec[11]/cellMin[11]);

//--------------------------------------------------//
// 		         Points			          //
//--------------------------------------------------//
//+
Point(1) = {0, -yh, 0, 1.0};
//+
Point(2) = {0, -yh, -z1, 1.0};
//+
Point(3) = {0, -y1, -zh, 1.0};
//+
Point(4) = {0, y1, -zh, 1.0};
//+
Point(5) = {0, yh, -z1, 1.0};
//+
Point(6) = {0, yh, 0, 1.0};

// First surrounding mesh structure
//+
Point(7) = {0, -yInn, 0, 1.0};
//+
Point(8) = {0, -yInn, -z1, 1.0};
//+
Point(9) = {0, -yd2, -zd2, 1.0};
//+
Point(10) = {0, -y1, -zInn, 1.0};
//+
Point(11) = {0, y1, -zInn, 1.0};
//+
Point(12) = {0, yd2, -zd2, 1.0};
//+
Point(13) = {0, yInn, -z1, 1.0};
//+
Point(14) = {0, yInn, 0, 1.0};

// Second surrounding mesh structure
//+
Point(15) = {0, -yOut, 0, 1.0};
//+
Point(16) = {0, -yOut, -z1, 1.0};
//+
Point(17) = {0, -y1, -zOut, 1.0};
//+
Point(18) = {0, y1, -zOut, 1.0};
//+
Point(19) = {0, yOut, -z1, 1.0};
//+
Point(20) = {0, yOut, 0, 1.0};

// Outer borders for the domain
//+
Point(21) = {0, -ySt, 0, 1.0};
//+
Point(22) = {0, -ySt, -z1, 1.0};
//+
Point(23) = {0, -ySt, -d, 1.0};
//+
Point(24) = {0, yEn, -d, 1.0};
//+
Point(25) = {0, yEn, -z1, 1.0};
//+
Point(26) = {0, yEn, 0, 1.0};
//+
Point(27) = {0, -y1, -d, 1.0};
//+
Point(28) = {0, y1, -d, 1.0};

// origo for circular arcs
//+
Point(29) = {0, -Oy, -Oz, 1.0};
//+
Point(30) = {0, Oy, -Oz, 1.0};

// Points of intersection with diagonal and arcs
//+
Point(31) = {0, -yd1, -zd1, 1.0};
//+
Point(32) = {0, yd1, -zd1, 1.0};
//+
Point(33) = {0, -yd3, -zd3, 1.0};
//+
Point(34) = {0, yd3, -zd3, 1.0};
//+
Point(35) = {0, -yd4, -zd4, 1.0};
//+
Point(36) = {0, yd4, -zd4, 1.0};
//+
// Horizontal support for diagonals
Point(37) = {0, -ySt, -zd3, 1.0};
//+
Point(38) = {0, yEn, -zd3, 1.0};
//+

//--------------------------------------------------//
// 		       		Lines						    //
//--------------------------------------------------//
//+
Line(1) = {2, 1};
//+
Circle(2) = {31, 29, 2};
//+
Circle(3) = {3, 29, 31};
//+
Line(4) = {3, 4};
//+
Circle(5) = {4, 30, 32};
//+
Circle(6) = {32, 30, 5};
//+
Line(7) = {5, 6};
//+
Line(8) = {8, 7};
//+
Circle(9) = {9, 29, 8};
//+
Circle(10) = {10, 29, 9};
//+
Line(11) = {10, 11};
//+
Circle(12) = {11, 30, 12};
//+
Circle(13) = {12, 30, 13};
//+
Line(14) = {13, 14};
//+
Line(15) = {16, 15};
//+
Circle(16) = {33, 29, 16};
//+
Circle(17) = {17, 29, 33};
//+
Line(18) = {17, 18};
//+
Circle(19) = {18, 30, 34};
//+
Circle(20) = {34, 30, 19};
//+
Line(21) = {19, 20};
//+
Line(22) = {1, 7};
//+
Line(23) = {2, 8};
//+
Line(24) = {31, 9};
//+
Line(25) = {3, 10};
//+
Line(26) = {4, 11};
//+
Line(27) = {32, 12};
//+
Line(28) = {5, 13};
//+
Line(29) = {6, 14};
//+
Line(30) = {7, 15};
//+
Line(31) = {8, 16};
//+
Line(32) = {9, 33};
//+
Line(33) = {10, 17};
//+
Line(34) = {11, 18};
//+
Line(35) = {12, 34};
//+
Line(36) = {13, 19};
//+
Line(37) = {14, 20};
//+
Line(38) = {15, 21};
//+
Line(39) = {16, 22};
//+
Line(40) = {33, 37};
//+
Line(41) = {33, 35};
//+
Line(42) = {17, 27};
//+
Line(43) = {18, 28};
//+
Line(44) = {34, 36};
//+
Line(45) = {34, 38};
//+
Line(46) = {19, 25};
//+
Line(47) = {20, 26};
//+
Line(48) = {23, 37};
//+
Line(49) = {37, 22};
//+
Line(50) = {22, 21};
//+
Line(51) = {24, 38};
//+
Line(52) = {38, 25};
//+
Line(53) = {25, 26};
//+
Line(54) = {35, 23};
//+
Line(55) = {27, 35};
//+
Line(56) = {27, 28};
//+
Line(57) = {28, 36};
//+
Line(58) = {36, 24};

//////////////////////////////////////////////
//	Set number of elements along the lines	//
//////////////////////////////////////////////
//+
// 0 = Surface to bottom inlet side
Transfinite Curve {38, 39, 40, 54} = nCells[0]-1 Using Progression r_exp[0];
//+
// 1 = Outer subdomain
Transfinite Curve {30, 31, 32, 33, 34, 35, 36, 37} = nCells[1]-1 Using Progression r_exp[1];
//+
// 2 = Inner subdomain
Transfinite Curve {22, 23, 24, 25, 26, 27, 28, 29} = nCells[2]-1 Using Progression r_exp[2];
//+
// 3 = Surface to bottom outlet side
Transfinite Curve {47, 46, 45, 58} = nCells[3]-1 Using Progression r_exp[3];
//+
// 4 = Inlet to top of hull
Transfinite Curve {1, 8, 15, 50} = nCells[4]-1 Using Progression r_exp[4];
//+
// 5 = Inlet to corner of hull
Transfinite Curve {2, 9, 16, 49} = nCells[5] Using Progression r_exp[5];
//+
// 6 = bottom to hull inlet side
Transfinite Curve {3, 10, 17, 55} = nCells[6] Using Progression r_exp[6];
//+
// 7 = bottom to hull
Transfinite Curve {4, 11, 18, 56} = nCells[7] Using Progression r_exp[7];
//+
// 8 = bottom to hull outlet side
Transfinite Curve {5, 12, 19, 57} = nCells[8] Using Progression r_exp[8];
//+
// 9 = Outlet to corner of hull
Transfinite Curve {6, 13, 20, 52} = nCells[9] Using Progression r_exp[9];
//+
// 10 = Outlet to top of hull
Transfinite Curve {7, 14, 21, 53} = nCells[10]-1 Using Progression r_exp[10];
//+
// 11 = Inlet to outlet
Transfinite Curve {48, 41, 42, 43, 44, 51} = nCells[11] Using Progression r_exp[11];

//--------------------------------------------------//
// 		       Set Surfaces			   				//
//--------------------------------------------------//
//+
Curve Loop(1) = {22, -8, -23, 1};
Plane Surface(1) = {1};
Transfinite Surface {1};
Recombine Surface {1};
//+
Curve Loop(2) = {23, -9, -24, 2};
Plane Surface(2) = {2};
Transfinite Surface {2};
Recombine Surface {2};
//+
Curve Loop(3) = {3, 24, -10, -25};
Plane Surface(3) = {3};
Transfinite Surface {3};
Recombine Surface {3};
//+
Curve Loop(4) = {4, 26, -11, -25};
Plane Surface(4) = {4};
Transfinite Surface {4};
Recombine Surface {4};
//+
Curve Loop(5) = {5, 27, -12, -26};
Plane Surface(5) = {5};
Transfinite Surface {5};
Recombine Surface {5};
//+
Curve Loop(6) = {6, 28, -13, -27};
Plane Surface(6) = {6};
Transfinite Surface {6};
Recombine Surface {6};
//+
Curve Loop(7) = {29, -14, -28, 7};
Plane Surface(7) = {7};
Transfinite Surface {7};
Recombine Surface {7};
//+
Curve Loop(8) = {30, -15, -31, 8};
Plane Surface(8) = {8};
Transfinite Surface {8};
Recombine Surface {8};
//+
Curve Loop(9) = {9, 31, -16, -32};
Plane Surface(9) = {9};
Transfinite Surface {9};
Recombine Surface {9};
//+
Curve Loop(10) = {10, 32, -17, -33};
Plane Surface(10) = {10};
Transfinite Surface {10};
Recombine Surface {10};
//+
Curve Loop(11) = {11, 34, -18, -33};
Plane Surface(11) = {11};
Transfinite Surface {11};
Recombine Surface {11};
//+
Curve Loop(12) = {12, 35, -19, -34};
Plane Surface(12) = {12};
Transfinite Surface {12};
Recombine Surface {12};
//+
Curve Loop(13) = {13, 36, -20, -35};
Plane Surface(13) = {13};
Transfinite Surface {13};
Recombine Surface {13};
//+
Curve Loop(14) = {14, 37, -21, -36};
Plane Surface(14) = {14};
Transfinite Surface {14};
Recombine Surface {14};
//+
Curve Loop(15) = {15, 38, -50, -39};
Plane Surface(15) = {15};
Transfinite Surface {15};
Recombine Surface {15};
//+
Curve Loop(16) = {16, 39, -49, -40};
Plane Surface(16) = {16};
Transfinite Surface {16};
Recombine Surface {16};
//+
Curve Loop(17) = {40, -48, -54, -41};
Plane Surface(17) = {17};
Transfinite Surface {17};
Recombine Surface {17};
//+
Curve Loop(18) = {17, 41, -55, -42};
Plane Surface(18) = {18};
Transfinite Surface {18};
Recombine Surface {18};
//+
Curve Loop(19) = {18, 43, -56, -42};
Plane Surface(19) = {19};
Transfinite Surface {19};
Recombine Surface {19};
//+
Curve Loop(20) = {19, 44, -57, -43};
Plane Surface(20) = {20};
Transfinite Surface {20};
Recombine Surface {20};
//+
Curve Loop(21) = {45, -51, -58, -44};
Plane Surface(21) = {21};
Transfinite Surface {21};
Recombine Surface {21};
//+
Curve Loop(22) = {20, 46, -52, -45};
Plane Surface(22) = {22};
Transfinite Surface {22};
Recombine Surface {22};
//+
Curve Loop(23) = {21, 47, -53, -46};
Plane Surface(23) = {23};
Transfinite Surface {23};
Recombine Surface {23};


//--------------------------------------------------//
// 		       Extrude Surfaces			    			//
//--------------------------------------------------//
//+
Extrude {-0.1, 0, 0} {
  Surface{1};	Surface{2};		Surface{3};		Surface{4};
  Surface{5};	Surface{6};		Surface{7};		Surface{8};
  Surface{9};	Surface{10};	Surface{11};	Surface{12};
  Surface{13};	Surface{14};	Surface{15};	Surface{16};
  Surface{17};	Surface{18};	Surface{19};	Surface{20};
  Surface{21};	Surface{22};	Surface{23};
  Layers {1};
  Recombine;
}

//--------------------------------------------------//
// 		    Classify Boundaries			            //
//--------------------------------------------------//
//+
Physical Surface("frontAndBack", 565) = {15, 388, 16, 410, 17, 432, 8, 234, 9, 256, 10, 278, 18, 454, 1, 80, 2, 102, 3, 124, 4, 146, 11, 300, 19, 476, 5, 168, 12, 322, 20, 498, 6, 190, 13, 344, 7, 212, 14, 366, 23, 564, 22, 542, 21, 520};
//+
Physical Surface("inlet", 566) = {383, 405, 423};
//+
Physical Surface("outlet", 567) = {559, 537, 511};
//+
Physical Surface("top", 568) = {555, 357, 199, 67, 221, 379};
//+
Physical Surface("bottom", 569) = {515, 493, 471, 449, 427};
//+
Physical Surface("hull", 570) = {211, 177, 155, 133, 111, 101, 79};


//------------------------------------------//
//	Create Volume	                          //
//------------------------------------------//
//+
Physical Volume("internalField", 571) = {
     1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
    11, 12, 13, 14, 15, 16, 17, 18,
    19, 20, 21, 22, 23
};

//------------------------------------------//
//	Write out mesh specific data to file	//
//------------------------------------------//
//+
nCells_tot =  nCells[0]*nCells[4]
            + nCells[0]*nCells[5]
            + nCells[0]*nCells[11]
            + nCells[1]*nCells[4]
            + nCells[1]*nCells[5]
            + nCells[1]*nCells[6]
            + nCells[1]*nCells[7]
            + nCells[1]*nCells[8]
            + nCells[1]*nCells[9]
            + nCells[1]*nCells[10]
            + nCells[2]*nCells[4]
            + nCells[2]*nCells[5]
            + nCells[2]*nCells[6]
            + nCells[2]*nCells[7]
            + nCells[2]*nCells[8]
            + nCells[2]*nCells[9]
            + nCells[2]*nCells[10]
            + nCells[3]*nCells[9]
            + nCells[3]*nCells[10]
            + nCells[3]*nCells[11]
            + nCells[11]*nCells[6]
            + nCells[11]*nCells[7]
            + nCells[11]*nCells[8];

Printf("Computational Domain:") > "mesh.dat";
Printf("") >> "mesh.dat";
Printf("///////////////////////////////////////////////") >> "mesh.dat";
Printf("//      Indexes:                             //") >> "mesh.dat";
Printf("//  0 = Surface to bottom inlet side         //") >> "mesh.dat";
Printf("//  1 = Outer subdomain                      //") >> "mesh.dat";
Printf("//  2 = Inner subdomain                      //") >> "mesh.dat";
Printf("//  3 = Surface to bottom outlet side        //") >> "mesh.dat";
Printf("//  4 = Inlet to top of hull                 //") >> "mesh.dat";
Printf("//  5 = Inlet to corner of hull              //") >> "mesh.dat";
Printf("//  6 = bottom to corner of hull inlet side  //") >> "mesh.dat";
Printf("//  7 = bottom to hull                       //") >> "mesh.dat";
Printf("//  8 = bottom to corner of hull outlet side //") >> "mesh.dat";
Printf("//  9 = Outlet to corner of hull             //") >> "mesh.dat";
Printf("// 10 = Outlet to top of hull                //") >> "mesh.dat";
Printf("// 11 = Inlet to outlet                      //") >> "mesh.dat";
Printf("///////////////////////////////////////////////") >> "mesh.dat";

Printf("") >> "mesh.dat";

Printf("___| Smallest cell length | Largest cell length | Expansion ratio | Length of line | Number of cells |") >> "mesh.dat";
Printf(" 0 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[0], cellMax[0], r_exp[0], L_vec[0], nCells[0]) >> "mesh.dat";
Printf(" 1 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[1], cellMax[1], r_exp[1], L_vec[1], nCells[1]) >> "mesh.dat";
Printf(" 2 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[2], cellMax[2], r_exp[2], L_vec[2], nCells[2]) >> "mesh.dat";
Printf(" 3 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[3], cellMax[3], r_exp[3], L_vec[3], nCells[3]) >> "mesh.dat";
Printf(" 4 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[4], cellMax[4], r_exp[4], L_vec[4], nCells[4]) >> "mesh.dat";
Printf(" 5 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[5], cellMax[5], r_exp[5], L_vec[5], nCells[5]) >> "mesh.dat";
Printf(" 6 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[6], cellMax[6], r_exp[6], L_vec[6], nCells[6]) >> "mesh.dat";
Printf(" 7 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[7], cellMax[7], r_exp[7], L_vec[7], nCells[7]) >> "mesh.dat";
Printf(" 8 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[8], cellMax[8], r_exp[8], L_vec[8], nCells[8]) >> "mesh.dat";
Printf(" 9 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[9], cellMax[9], r_exp[9], L_vec[9], nCells[9]) >> "mesh.dat";
Printf("10 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[10], cellMax[10], r_exp[10], L_vec[10], nCells[10]) >> "mesh.dat";
Printf("11 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[11], cellMax[11], r_exp[11], L_vec[11], nCells[11]) >> "mesh.dat";

Printf("") >> "mesh.dat";

Printf("Total number of grid cells:") >> "mesh.dat";
Printf("nCells = %12.2f", nCells_tot) >> "mesh.dat";

//--------------------------------------------//
//	Write out CFD model specific data to file //
//--------------------------------------------//
//+
rho = 1025;
CFL_max = {1, 0.75, 0.5, 0.25};               // Courant-Friedrichs-Lewy
nu_s = 1.3604e-6;
Re_s = Breadth * currentVelocity / nu_s;

nu_CFDs = currentVelocity*Breadth*scaleRatio / Re_s;
nu_CFD_Re1e5 = currentVelocity*Breadth*scaleRatio / 1e5;

k_inlet = 1.5 * (currentVelocity * turbulenceIntensity)^2;
k_5perc = 1.5 * (currentVelocity * 0.05)^2;
k_10perc = 1.5 * (currentVelocity * 0.1)^2;
Omega_inlet = Sqrt(k_inlet) /  (0.09^(0.25)*0.07*zh);
Omega_5perc = Sqrt(k_5perc) /  (0.09^(0.25)*0.07*zh);
Omega_10perc = Sqrt(k_10perc) /  (0.09^(0.25)*0.07*zh);

dt_max[0] = CFL_max[0] * fine / currentVelocity;
dt_max[1] = CFL_max[1] * fine / currentVelocity;
dt_max[2] = CFL_max[2] * fine / currentVelocity;
dt_max[3] = CFL_max[3] * fine / currentVelocity;

Printf("") >> "mesh.dat";

Printf("Adjusted kinematic viscosity for CFD purposes:") >> "mesh.dat";
Printf("Re = Re_s") >> "mesh.dat";
Printf("nu_CFD = %1.4e", nu_CFDs) >> "mesh.dat";

Printf("Re = 1e5") >> "mesh.dat";
Printf("nu_CFD = %1.4e", nu_CFD_Re1e5) >> "mesh.dat";


Printf("") >> "mesh.dat";

Printf("Turbulent kinetic energy and specific turbulent dissipation:") >> "mesh.dat";
Printf("k and Omega:") >> "mesh.dat";
Printf("k_inlet = 1.5 (U_inf * I)^2 = %6.7f", k_inlet) >> "mesh.dat";
Printf("k_5percent = 1.5 (U_inf * I)^2 = %6.7f", k_5perc) >> "mesh.dat";
Printf("k_10percent = 1.5 (U_inf * I)^2 = %6.7f", k_10perc) >> "mesh.dat";
Printf("Omega_inlet = k_inlet^(0.5) / (C_mu^(0.25) * 0.07 * L) = %6.7f", Omega_inlet) >> "mesh.dat";
Printf("Omega_5percent = k_5perc^(0.5) / (C_mu^(0.25) * 0.07 * L) = %6.7f", Omega_5perc) >> "mesh.dat";
Printf("Omega_10percent = k_10perc^(0.5) / (C_mu^(0.25) * 0.07 * L) = %6.7f", Omega_10perc) >> "mesh.dat";

Printf("") >> "mesh.dat";

Printf("Largest possible timestep:") >> "mesh.dat";
Printf("C_1    = %1.2e", dt_max[0]) >> "mesh.dat";
Printf("C_0.75 = %1.2e", dt_max[1]) >> "mesh.dat";
Printf("C_0.5  = %1.2e", dt_max[2]) >> "mesh.dat";
Printf("C_0.25 = %1.2e", dt_max[3]) >> "mesh.dat";

//--------------------------------------------//
//	       Engage Meshing In GUI	          //
//--------------------------------------------//
//+
Mesh 3;





