//----------------------------------------------//
//	Helpfull function for deciding mesh size	  //
//----------------------------------------------//

// a1: mesh size of the first cell 
// aN: mesh size of the last cell
Macro CalExpansionRatio
   NumGrid = Log(1 - Sn * (1-q) / a0) / Log(q);
   NumGrid = Round(NumGrid) + 1;
   aN	   = a0 * q^(NumGrid - 1);
   NumNode = NumGrid;   
Return

//--------------------------------------------------//
//  GMSH project created by Brage Møller-Pettersen  //
//--------------------------------------------------//

//--------------------------------------------------------------------//
// 	Original vessel used as a mockup model:				    		  //
// 	UT 5400 WP vessel range Windfarm Service Operation Vessel (SOV)	  //
//--------------------------------------------------------------------//
LoA					        = 82.0;	        // [m]					  //
L_pp				        = 72.1;	        // [m]					  //
Breadth				        = 18.0;         // [m]					  //
Draft				        = 5.0;          // [m]					  //
thrusterDiameter	        = 2.5;	        // [m]					  //
currentVelocity             = 1.0;          // [m/s]                  //
turbulenceIntensity         = 0.1;          // [%]                    //
scaleRatio			        = 1/20;         // [-]					  //
//--------------------------------------------------------------------//

//////////////////////////////////////////
//	Coefficients Deciding Mesh Shape	//
//////////////////////////////////////////
//+
// DoT = Depth/Draft
DoT = 3;
// Radii of curvature in hull corners
Radii = 3;	                        // [m]	// Must be smaller than draft
k = Radii * scaleRatio;

//////////////////////////////////////
//	Outer points of the hull	    //
//////////////////////////////////////
//+
yh = Breadth * 0.5 * scaleRatio;
zh = Draft * scaleRatio;

////////////////////
//	Waterdepth	//
////////////////////
//+
d = DoT*zh;		clearance = d - zh;

pDia = thrusterDiameter * scaleRatio;
outerCoeff = clearance * 0.9;

//////////////////////////////////////
//	z coordinates from the top	    //
//////////////////////////////////////
//+
z1 = zh - k;
zOut = zh + outerCoeff;

//////////////////////////////////////
//	y coordinates from inlet	    //
//////////////////////////////////////
//+
ySt = yh * 4;			yEn = yh * 4.5;
yOut = yh + outerCoeff;
y1 = yh - k;

////////////////////////////////////////////
//	y & z coordinates for circular arcs   //
////////////////////////////////////////////
//+
Oy = y1;
Oz = z1;
rh = yh - Oy;
rOut = yOut - Oy;

////////////////////////////////////////////////////////////////////
//	y & z coordinates for the diagonal line intersecting arcs	  //
////////////////////////////////////////////////////////////////////
//+
yd1 = Oy + rh * Sqrt(2) * 0.5;		zd1 = Oz + rh * Sqrt(2) * 0.5;
yd3 = Oy + rOut * Sqrt(2) * 0.5;	zd3 = Oz + rOut * Sqrt(2) * 0.5;
yd4 = Oy + (d-z1) * Tan(Pi/4);		zd4 = d;

//////////////////////////////////////////
//	y & z coordinates for the propeller	//
//////////////////////////////////////////
//+
pRad = pDia * 0.5;
pCyl = pDia * 0.2;
pDistHull = pDia * 0.25;

pY = pCyl * 0.5;
pZtop = zh + pDistHull;
pZbottom = pZtop + pDia;

//////////////////////////////////////////////////////
//	y & z coordinates for the propeller boundaries	//
//////////////////////////////////////////////////////
//+
ypT1 = Oy + (pZtop-z1) * Sqrt(2) * 0.5;      zpT1 = Oz + (pZtop-z1) * Sqrt(2) * 0.5;
ypT2 = yh + pDistHull;

ypB1 = Oy + (pZbottom-z1) * Sqrt(2) * 0.5;   zpB1 = Oz + (pZbottom-z1) * Sqrt(2) * 0.5;
ypB2 = yh + pDistHull + pDia;

////////////////////////////////////////////////////////////////////
//			The Length Of Each Meshing Line			//
////////////////////////////////////////////////////////////////////
//+
L_vec = {
    ySt - yd3,                        //  0
    zOut - pZbottom,                  //  1
    pZbottom - pZtop,                 //  2
    pZtop - zh,                       //  3
    yEn - yOut,                       //  4
    z1,                               //  5
    Pi * 2 * k * 0.125,               //  6
    Pi * 2 * k * 0.125,               //  7
    y1 - pY,                          //  8
    pY * 2,                           //  9
    y1 - pY,                          // 10
    Pi*2*k*0.125,                     // 11
    Pi*2*k*0.125,                     // 12
    z1,                               // 13
    d - zd3                           // 14
};

//////////////////////////////////////////////////
//	Calculate Appropriate Number Of Elements	//
//////////////////////////////////////////////////
//+

//////////////////////////////////////////////////
//	Indexes:								  	//
//	 0 = surface from inlet to outer			//
//	 1 = outer subdomain						//
//	 2 = propeller			                    //
//	 3 = inner subdomain					    //
//   4 = surface from outlet to outer           //
//   5 = inlet to top of hull                   //
//	 6 = inlet to corner of hull				//
//	 7 = bottom to corner of hull, inlet side	//
//	 8 = bottom to hull	, inlet side			//
//   9 = bottom to hull, propeller              //
//  10 = bottom to hull, outlet side            //
//	11 = bottom to corner of hull, outlet side	//
//	12 = outlet to corner of hull				//
//	13 = outlet to top of hull					//
//	14 = inlet to outlet						//
//////////////////////////////////////////////////

// Define the smallest cell side length:

meshSizes = {       // nCells:      id:
        6.4e-3,     //    9966       0
        3.55e-3,    //   24965       1
        2.25e-3,    //   49467       2
        1.71e-3,    //   75149       3
        1.4e-3,     //  100366       4
        1.21e-3,    //  124897       5
        1.06e-3,    //  150130       6
        9.5e-4,     //  175046       7
        8.7e-4,     //  199205       8
        7.99e-4,    //  224759       9
        7.4e-4,     //  249700      10
        4.45e-4,    //  499418      11
        3.27e-4,    //  749689      12
        2.625e-4    // 1000411      13
};

fine = meshSizes[3];
semiFine = fine * 1.25;
coarseFine = fine * 1.5;
coarse = fine * 2;
veryCoarse = fine * 12;

/*  Suggestion 
    fine = meshSizes[];
    semiFine = fine * 1.1;
    coarseFine = semiFine * 1.1;
    coarse = coarseFine * 1.1;
    veryCoarse = coarse * 1.1;
*/

cellMin = {
    veryCoarse,   //  0               // Matches largest cell in 1
    semiFine,     //  1               // Matches largest cell in 2
    semiFine,     //  2               // Matches largest cell in 3
    fine,         //  3
    semiFine,     //  4               // Matches largest cell in 1
    fine,         //  5
    fine,         //  6
    fine,         //  7
    fine,         //  8
    fine,         //  9
    fine,         // 10
    fine,         // 11
    fine,         // 12
    coarseFine,   // 13
    fine          // 14
};
// Define expansion ratios
r_exp = {
    1.1,          //  0
    1.0125,       //  1
    1.0125,       //  2
    1.0125,       //  3
    1.1,          //  4
    1.025,        //  5
    1,            //  6
    1,            //  7
    1,            //  8
    1,            //  9
    1,            // 10
    1,            // 11
    1,            // 12
    1.025,        // 13
    1             // 14
};
// Preallocate vectors for largest grid cells and number of nodes
cellMax = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
nCells = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

//	3 = inner subdomain
Sn = L_vec[3];
q = r_exp[3];
a0 = cellMin[3];
Call CalExpansionRatio;
cellMax[3] = aN;
nCells[3] = NumNode;

//	 2 = propeller
Sn = L_vec[2];
q = r_exp[2];
a0 = cellMax[3];              cellMin[2] = cellMax[3];
Call CalExpansionRatio;
cellMax[2] = aN;
nCells[2] = NumNode;

//	1 = outer subdomain
Sn = L_vec[1];
q = r_exp[1];
a0 = cellMax[2];              cellMin[1] = cellMax[2];
Call CalExpansionRatio;
cellMax[1] = aN;
nCells[1] = NumNode;

//	0 = surface from inlet to outer
Sn = L_vec[0];
q = r_exp[0];
a0 = cellMax[1]*r_exp[0];    cellMin[0] = cellMax[1]*r_exp[0];
Call CalExpansionRatio;
cellMax[0] = aN;
nCells[0] = NumNode;

//   4 = surface from outlet to outer
Sn = L_vec[4];
q = r_exp[4];
a0 = cellMax[1];             cellMin[4] = cellMax[1];
Call CalExpansionRatio;
cellMax[4] = aN;
nCells[4] = NumNode;

//   5 = inlet to top of hull
Sn = L_vec[5];
q = r_exp[5];
a0 = cellMin[5];
Call CalExpansionRatio;
cellMax[5] = aN;
nCells[5] = NumNode;

//	 6 = inlet to corner of hull
cellMax[6] =  cellMin[6];
nCells[6] = Round(L_vec[6]/cellMin[6]);

//	 7 = bottom to corner of hull, inlet side
cellMax[7] =  cellMin[7];
nCells[7] = Round(L_vec[7]/cellMin[7]);

//	 8 = bottom to hull	, inlet side
cellMax[8] =  cellMin[8];
nCells[8] = Round(L_vec[8]/cellMin[8]);

//   9 = bottom to hull, propeller
cellMax[9] =  cellMin[9];
nCells[9] = Round(L_vec[9]/cellMin[9]);

//  10 = bottom to hull, outlet side
cellMax[10] =  cellMin[10];
nCells[10] = Round(L_vec[10]/cellMin[10]);

//	11 = bottom to corner of hull, outlet side
cellMax[11] =  cellMin[11];
nCells[11] = Round(L_vec[11]/cellMin[11]);

//	12 = outlet to corner of hull
cellMax[12] =  cellMin[12];
nCells[12] = Round(L_vec[12]/cellMin[12]);

//	13 = outlet to top of hull
Sn = L_vec[13];
q = r_exp[13];
a0 = cellMin[13];
Call CalExpansionRatio;
cellMax[13] = aN;
nCells[13] = NumNode;

//	14 = inlet to outlet
cellMin[14] = cellMax[1];
cellMax[14] =  cellMin[14];
nCells[14] = Round(L_vec[14]/cellMin[14]);

/* Sn = L_vec[14];
q = r_exp[14];
a0 = cellMax[1];      cellMin[14] = cellMax[1];
Call CalExpansionRatio;
cellMax[14] = aN;
nCells[14] = NumNode; */

//--------------------------------------------------//
// 		         Points			                    //
//--------------------------------------------------//
//+
Point(1) = {0, -yh, 0, 1.0};
//+
Point(2) = {0, -yh, -z1, 1.0};
//+
Point(3) = {0, -y1, -zh, 1.0};
//+
Point(4) = {0, y1, -zh, 1.0};
//+
Point(5) = {0, yh, -z1, 1.0};
//+
Point(6) = {0, yh, 0, 1.0};

// Second surrounding mesh structure
//+
Point(7) = {0, -yOut, 0, 1.0};
//+
Point(8) = {0, -yOut, -z1, 1.0};
//+
Point(9) = {0, -y1, -zOut, 1.0};
//+
Point(10) = {0, y1, -zOut, 1.0};
//+
Point(11) = {0, yOut, -z1, 1.0};
//+
Point(12) = {0, yOut, 0, 1.0};

// Outer borders for the domain
//+
Point(13) = {0, -ySt, 0, 1.0};
//+
Point(14) = {0, -ySt, -z1, 1.0};
//+
Point(15) = {0, -ySt, -d, 1.0};
//+
Point(16) = {0, yEn, -d, 1.0};
//+
Point(17) = {0, yEn, -z1, 1.0};
//+
Point(18) = {0, yEn, 0, 1.0};
//+
Point(19) = {0, -y1, -d, 1.0};
//+
Point(20) = {0, y1, -d, 1.0};

// origo for circular arcs
//+
Point(21) = {0, -Oy, -Oz, 1.0};
//+
Point(22) = {0, Oy, -Oz, 1.0};

// Points of intersection with diagonal and arcs
//+
Point(23) = {0, -yd1, -zd1, 1.0};
//+
Point(24) = {0, yd1, -zd1, 1.0};
//+
Point(25) = {0, -yd3, -zd3, 1.0};
//+
Point(26) = {0, yd3, -zd3, 1.0};
//+
Point(27) = {0, -yd4, -zd4, 1.0};
//+
Point(28) = {0, yd4, -zd4, 1.0};
//+
// Horizontal support for diagonals
Point(29) = {0, -ySt, -zd3, 1.0};
//+
Point(30) = {0, yEn, -zd3, 1.0};
//+
//  Propeller
Point(31) = {0, -pY, -pZtop, 1.0};
//+
Point(32) = {0, pY, -pZtop, 1.0};
//+
Point(33) = {0, pY, -pZbottom, 1.0};
//+
Point(34) = {0, -pY, -pZbottom, 1.0};
//+
//    Propeller boundary points
Point(35) = {0, -y1, -pZtop, 1.0};
Point(36) = {0, -ypT1, -zpT1, 1.0};
Point(37) = {0, -ypT2, -z1, 1.0};
Point(38) = {0, -ypT2, 0, 1.0};
//+
Point(39) = {0, y1, -pZtop, 1.0};
Point(40) = {0, ypT1, -zpT1, 1.0};
Point(41) = {0, ypT2, -z1, 1.0};
Point(42) = {0, ypT2, 0, 1.0};
//+
Point(43) = {0, -y1, -pZbottom, 1.0};
Point(44) = {0, -ypB1, -zpB1, 1.0};
Point(45) = {0, -ypB2, -z1, 1.0};
Point(46) = {0, -ypB2, 0, 1.0};
//+
Point(47) = {0, y1, -pZbottom, 1.0};
Point(48) = {0, ypB1, -zpB1, 1.0};
Point(49) = {0, ypB2, -z1, 1.0};
Point(50) = {0, ypB2, 0, 1.0};
//+
Point(51) = {0, -pY, -zh, 1.0};
Point(52) = {0, -pY, -zOut, 1.0};
Point(53) = {0, -pY, -d, 1.0};
//+
Point(54) = {0, pY, -zh, 1.0};
Point(55) = {0, pY, -zOut, 1.0};
Point(56) = {0, pY, -d, 1.0};

//--------------------------------------------------//
// 		       		Lines						    //
//--------------------------------------------------//
//+
Line(1) = {2, 1};
//+
Circle(2) = {23, 21, 2};
//+
Circle(3) = {3, 21, 23};
//+
Line(4) = {51, 3};
//+
Line(5) = {51, 54};
//+
Line(6) = {54, 4};
//+
Circle(7) = {4, 22, 24};
//+
Circle(8) = {24, 22, 5};
//+
Line(9) = {5, 6};
//+
Line(10) = {37, 38};
//+
Circle(11) = {36, 21, 37};
//+
Circle(12) = {35, 21, 36};
//+
Line(13) = {31, 35};
//+
Line(14) = {31, 32};
//+
Line(15) = {32, 39};
//+
Circle(16) = {39, 22, 40};
//+
Circle(17) = {40, 22, 41};
//+
Line(18) = {41, 42};
//+
Line(19) = {45, 46};
//+
Circle(20) = {44, 21, 45};
//+
Circle(21) = {43, 21, 44};
//+
Line(22) = {34, 43};
//+
Line(23) = {34, 33};
//+
Line(24) = {33, 47};
//+
Circle(25) = {47, 22, 48};
//+
Circle(26) = {48, 22, 49};
//+
Line(27) = {49, 50};
//+
Line(28) = {8, 7};
//+
Circle(29) = {25, 21, 8};
//+
Circle(30) = {9, 21, 25};
//+
Line(31) = {52, 9};
//+
Line(32) = {52, 55};
//+
Line(33) = {55, 10};
//+
Circle(34) = {10, 22, 26};
//+
Circle(35) = {26, 22, 11};
//+
Line(36) = {11, 12};
//+
Line(37) = {14, 13};
//+
Line(38) = {29, 14};
//+
Line(39) = {29, 15};
//+
Line(40) = {27, 15};
//+
Line(41) = {19, 27};
//+
Line(42) = {53, 19};
//+
Line(43) = {53, 56};
//+
Line(44) = {56, 20};
//+
Line(45) = {20, 28};
//+
Line(46) = {28, 16};
//+
Line(47) = {30, 16};
//+
Line(48) = {30, 17};
//+
Line(49) = {17, 18};
//+
Line(50) = {1, 38};
//+
Line(51) = {2, 37};
//+
Line(52) = {23, 36};
//+
Line(53) = {3, 35};
//+
Line(54) = {51, 31};
//+
Line(55) = {54, 32};
//+
Line(56) = {4, 39};
//+
Line(57) = {24, 40};
//+
Line(58) = {5, 41};
//+
Line(59) = {6, 42};
//+
Line(60) = {38, 46};
//+
Line(61) = {37, 45};
//+
Line(62) = {36, 44};
//+
Line(63) = {35, 43};
//+
Line(64) = {31, 34};
//+
Line(65) = {32, 33};
//+
Line(66) = {39, 47};
//+
Line(67) = {40, 48};
//+
Line(68) = {41, 49};
//+
Line(69) = {42, 50};
//+
Line(70) = {46, 7};
//+
Line(71) = {45, 8};
//+
Line(72) = {44, 25};
//+
Line(73) = {43, 9};
//+
Line(74) = {34, 52};
//+
Line(75) = {33, 55};
//+
Line(76) = {47, 10};
//+
Line(77) = {48, 26};
//+
Line(78) = {49, 11};
//+
Line(79) = {50, 12};
//+
Line(80) = {7, 13};
//+
Line(81) = {8, 14};
//+
Line(82) = {25, 29};
//+
Line(83) = {25, 27};
//+
Line(84) = {9, 19};
//+
Line(85) = {52, 53};
//+
Line(86) = {55, 56};
//+
Line(87) = {10, 20};
//+
Line(88) = {26, 28};
//+
Line(89) = {26, 30};
//+
Line(90) = {11, 17};
//+
Line(91) = {12, 18};

//////////////////////////////////////////////
//	Set number of elements along the lines	//
//////////////////////////////////////////////
//+
//	 0 = surface from inlet to outer
Transfinite Curve {80, 81, 82, 40} = nCells[0]-1 Using Progression r_exp[0];
//+
//	 1 = outer subdomain
Transfinite Curve {70, 71, 72, 73, 74, 75, 76, 77, 78, 79} = nCells[1] Using Progression r_exp[1];
//+
//	 2 = propeller
Transfinite Curve {60, 61, 62, 63, 64, 65, 66, 67, 68, 69} = nCells[2]-1 Using Progression r_exp[2];
//+
//	 3 = inner subdomain
Transfinite Curve {50, 51, 52, 53, 54, 55, 56, 57, 58, 59} = nCells[3]-1 Using Progression r_exp[3];
//+
//   4 = surface from outlet to outer
Transfinite Curve {91, 90, 89, 46} = nCells[4]-1 Using Progression r_exp[4];
//+
//   5 = inlet to top of hull
Transfinite Curve {1, 10, 19, 28, 37} = nCells[5]-1 Using Progression r_exp[5];
//+
//	 6 = inlet to corner of hull
Transfinite Curve {2, 11, 20, 29, 38} = nCells[6] Using Progression r_exp[6];
//+
//	 7 = bottom to corner of hull, inlet side
Transfinite Curve {3, 12, 21, 30, 41} = nCells[7] Using Progression r_exp[7];
//+
//	 8 = bottom to hull	, inlet side
Transfinite Curve {4, 13, 22, 31, 42} = nCells[8] Using Progression r_exp[8];
//+
//   9 = bottom to hull, propeller
Transfinite Curve {5, 14, 23, 32, 43} = nCells[9] Using Progression r_exp[9];
//+
//  10 = bottom to hull, outlet side
Transfinite Curve {6, 15, 24, 33, 44} = nCells[10] Using Progression r_exp[10];
//+
//	11 = bottom to corner of hull, outlet side
Transfinite Curve {7, 16, 25, 34, 45} = nCells[11] Using Progression r_exp[11];
//+
//	12 = outlet to corner of hull
Transfinite Curve {8, 17, 26, 35, 48} = nCells[12] Using Progression r_exp[12];
//+
//	13 = outlet to top of hull
Transfinite Curve {9, 18, 27, 36, 49} = nCells[13]-1 Using Progression r_exp[13];
//+
//	14 = inlet to outlet
Transfinite Curve {47, 88, 87, 86, 85, 84, 83, 39} = nCells[14] Using Progression r_exp[14];

//--------------------------------------------------//
// 		       Set Surfaces			   				//
//--------------------------------------------------//
//+
Curve Loop(1) = {1, 50, -10, -51};
Plane Surface(1) = {1};
Transfinite Surface {1};
Recombine Surface {1};
//+
Curve Loop(2) = {51, -11, -52, 2};
Plane Surface(2) = {2};
Transfinite Surface {2};
Recombine Surface {2};
//+
Curve Loop(3) = {3, 52, -12, -53};
Plane Surface(3) = {3};
Transfinite Surface {3};
Recombine Surface {3};
//+
Curve Loop(4) = {4, 53, -13, -54};
Plane Surface(4) = {4};
Transfinite Surface {4};
Recombine Surface {4};
//+
Curve Loop(5) = {5, 55, -14, -54};
Plane Surface(5) = {5};
Transfinite Surface {5};
Recombine Surface {5};
//+
Curve Loop(6) = {6, 56, -15, -55};
Plane Surface(6) = {6};
Transfinite Surface {6};
Recombine Surface {6};
//+
Curve Loop(7) = {7, 57, -16, -56};
Plane Surface(7) = {7};
Transfinite Surface {7};
Recombine Surface {7};
//+
Curve Loop(8) = {8, 58, -17, -57};
Plane Surface(8) = {8};
Transfinite Surface {8};
Recombine Surface {8};
//+
Curve Loop(9) = {59, -18, -58, 9};
Plane Surface(9) = {9};
Transfinite Surface {9};
Recombine Surface {9};
//+
Curve Loop(10) = {69, -27, -68, 18};
Plane Surface(10) = {10};
Transfinite Surface {10};
Recombine Surface {10};
//+
Curve Loop(11) = {60, -19, -61, 10};
Plane Surface(11) = {11};
Transfinite Surface {11};
Recombine Surface {11};
//+
Curve Loop(12) = {61, -20, -62, 11};
Plane Surface(12) = {12};
Transfinite Surface {12};
Recombine Surface {12};
//+
Curve Loop(13) = {12, 62, -21, -63};
Plane Surface(13) = {13};
Transfinite Surface {13};
Recombine Surface {13};
//+
Curve Loop(14) = {13, 63, -22, -64};
Plane Surface(14) = {14};
Transfinite Surface {14};
Recombine Surface {14};
//+
Curve Loop(15) = {14, 65, -23, -64};
Plane Surface(15) = {15};
Transfinite Surface {15};
Recombine Surface {15};
//+
Curve Loop(16) = {15, 66, -24, -65};
Plane Surface(16) = {16};
Transfinite Surface {16};
Recombine Surface {16};
//+
Curve Loop(17) = {16, 67, -25, -66};
Plane Surface(17) = {17};
Transfinite Surface {17};
Recombine Surface {17};
//+
Curve Loop(18) = {68, -26, -67, 17};
Plane Surface(18) = {18};
Transfinite Surface {18};
Recombine Surface {18};
//+
Curve Loop(19) = {70, -28, -71, 19};
Plane Surface(19) = {19};
Transfinite Surface {19};
Recombine Surface {19};
//+
Curve Loop(20) = {71, -29, -72, 20};
Plane Surface(20) = {20};
Transfinite Surface {20};
Recombine Surface {20};
//+
Curve Loop(21) = {21, 72, -30, -73};
Plane Surface(21) = {21};
Transfinite Surface {21};
Recombine Surface {21};
//+
Curve Loop(22) = {22, 73, -31, -74};
Plane Surface(22) = {22};
Transfinite Surface {22};
Recombine Surface {22};
//+
Curve Loop(23) = {23, 75, -32, -74};
Plane Surface(23) = {23};
Transfinite Surface {23};
Recombine Surface {23};
//+
Curve Loop(24) = {24, 76, -33, -75};
Plane Surface(24) = {24};
Transfinite Surface {24};
Recombine Surface {24};
//+
Curve Loop(25) = {25, 77, -34, -76};
Plane Surface(25) = {25};
Transfinite Surface {25};
Recombine Surface {25};
//+
Curve Loop(26) = {78, -35, -77, 26};
Plane Surface(26) = {26};
Transfinite Surface {26};
Recombine Surface {26};
//+
Curve Loop(27) = {79, -36, -78, 27};
Plane Surface(27) = {27};
Transfinite Surface {27};
Recombine Surface {27};
//+
Curve Loop(28) = {80, -37, -81, 28};
Plane Surface(28) = {28};
Transfinite Surface {28};
Recombine Surface {28};
//+
Curve Loop(29) = {81, -38, -82, 29};
Plane Surface(29) = {29};
Transfinite Surface {29};
Recombine Surface {29};
//+
Curve Loop(30) = {82, 39, -40, -83};
Plane Surface(30) = {30};
Transfinite Surface {30};
Recombine Surface {30};
//+
Curve Loop(31) = {30, 83, -41, -84};
Plane Surface(31) = {31};
Transfinite Surface {31};
Recombine Surface {31};
//+
Curve Loop(32) = {31, 84, -42, -85};
Plane Surface(32) = {32};
Transfinite Surface {32};
Recombine Surface {32};
//+
Curve Loop(33) = {32, 86, -43, -85};
Plane Surface(33) = {33};
Transfinite Surface {33};
Recombine Surface {33};
//+
Curve Loop(34) = {33, 87, -44, -86};
Plane Surface(34) = {34};
Transfinite Surface {34};
Recombine Surface {34};
//+
Curve Loop(35) = {34, 88, -45, -87};
Plane Surface(35) = {35};
Transfinite Surface {35};
Recombine Surface {35};
//+
Curve Loop(36) = {89, 47, -46, -88};
Plane Surface(36) = {36};
Transfinite Surface {36};
Recombine Surface {36};
//+
Curve Loop(37) = {90, -48, -89, 35};
Plane Surface(37) = {37};
Transfinite Surface {37};
Recombine Surface {37};
//+
Curve Loop(38) = {91, -49, -90, 36};
Plane Surface(38) = {38};
Transfinite Surface {38};
Recombine Surface {38};


//--------------------------------------------------//
// 		       Extrude Surfaces			    			//
//--------------------------------------------------//
//+
Extrude {-0.1, 0, 0} {
  Surface{1};	Surface{2};		Surface{3};		Surface{4};
  Surface{5};	Surface{6};		Surface{7};		Surface{8};
  Surface{9};	Surface{10};	Surface{11};	Surface{12};
  Surface{13};	Surface{14};	Surface{15};	Surface{16};
  Surface{17};	Surface{18};	Surface{19};	Surface{20};
  Surface{21};	Surface{22};	Surface{23};    Surface{24};
  Surface{25};	Surface{26};	Surface{27};	Surface{28};
  Surface{29};	Surface{30};	Surface{31};	Surface{32};
  Surface{33};	Surface{34};	Surface{35};	Surface{36};
  Surface{37};	Surface{38};
  Layers {1};
  Recombine;
}

//--------------------------------------------------//
// 		    Classify Boundaries			                  //
//--------------------------------------------------//
//+
Physical Surface("frontAndBack", 928) = {707, 28, 729, 29, 751, 30, 19, 509,
                                          20, 531, 21, 553, 31, 773, 12, 355,
                                          11, 333, 113, 1, 135, 2, 3, 157, 13,
                                          377, 4, 179, 399, 14, 22, 575, 32, 795,
                                          5, 201, 15, 421, 23, 597, 33, 817, 6,
                                          223, 16, 443, 24, 619, 34, 839, 245, 7,
                                          17, 465, 25, 641, 861, 35, 267, 8, 487, 18,
                                          26, 663, 37, 905, 883, 36, 289, 9, 311, 10,
                                          685, 27, 38, 927};
//+
Physical Surface("inlet", 929) = {742, 720, 698};
//+
Physical Surface("outlet", 930) = {874, 896, 918};
//+
Physical Surface("top", 931) = {914, 672, 298, 276, 104, 320, 496, 694};
//+
Physical Surface("bottom", 932) = {746, 768, 790, 812, 834, 856, 878};
//+
Physical Surface("hull", 933) = {288, 254, 232, 210, 188, 166, 144, 134, 100};
//+
Physical Surface("propInlet", 934) = {398};
//+
Physical Surface("propOutlet", 935) = {412};
//+
Physical Surface("propTopAndBottom", 936) = {196, 416};

//------------------------------------------//
//	Create Volume	                          //
//------------------------------------------//
//+
Physical Volume("internalField", 937) = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
                                        11, 12, 13, 14, 16, 17, 18, 19,
                                        20, 21, 22, 23, 24, 25, 26, 27,
                                        28, 29, 30, 31, 32, 33, 34, 35,
                                        36, 37, 38};

//------------------------------------------//
//	Write out mesh specific data to file	//
//------------------------------------------//

nCells_tot =  nCells[0]*nCells[5]
            + nCells[0]*nCells[6]
            + nCells[0]*nCells[14]
            + nCells[1]*nCells[5]
            + nCells[1]*nCells[6]
            + nCells[1]*nCells[7]
            + nCells[1]*nCells[8]
            + nCells[1]*nCells[9]
            + nCells[1]*nCells[10]
            + nCells[1]*nCells[11]
            + nCells[1]*nCells[12]
            + nCells[1]*nCells[13]
            + nCells[2]*nCells[5]
            + nCells[2]*nCells[6]
            + nCells[2]*nCells[7]
            + nCells[2]*nCells[8]
            + nCells[2]*nCells[9]
            + nCells[2]*nCells[10]
            + nCells[2]*nCells[11]
            + nCells[2]*nCells[12]
            + nCells[2]*nCells[13]
            + nCells[3]*nCells[5]
            + nCells[3]*nCells[6]
            + nCells[3]*nCells[7]
            + nCells[3]*nCells[8]
            + nCells[3]*nCells[9]
            + nCells[3]*nCells[10]
            + nCells[3]*nCells[11]
            + nCells[3]*nCells[12]
            + nCells[3]*nCells[13]
            + nCells[4]*nCells[13]
            + nCells[4]*nCells[12]
            + nCells[4]*nCells[14]
            + nCells[14]*nCells[7]
            + nCells[14]*nCells[8]
            + nCells[14]*nCells[9]
            + nCells[14]*nCells[10]
            + nCells[14]*nCells[11];

Printf("Computational Domain:") > "meshPropeller.dat";
Printf("") >> "meshPropeller.dat";
Printf("//////////////////////////////////////////////////") >> "meshPropeller.dat";
Printf("//	Indexes:								  	//") >> "meshPropeller.dat";
Printf("//	 0 = surface from inlet to outer			//") >> "meshPropeller.dat";
Printf("//	 1 = outer subdomain						//") >> "meshPropeller.dat";
Printf("//	 2 = propeller			                    //") >> "meshPropeller.dat";
Printf("//	 3 = inner subdomain					    //") >> "meshPropeller.dat";
Printf("//   4 = surface from outlet to outer           //") >> "meshPropeller.dat";
Printf("//   5 = inlet to top of hull                   //") >> "meshPropeller.dat";
Printf("//	 6 = inlet to corner of hull				//") >> "meshPropeller.dat";
Printf("//	 7 = bottom to corner of hull, inlet side	//") >> "meshPropeller.dat";
Printf("//	 8 = bottom to hull	, inlet side			//") >> "meshPropeller.dat";
Printf("//   9 = bottom to hull, propeller              //") >> "meshPropeller.dat";
Printf("//  10 = bottom to hull, outlet side            //") >> "meshPropeller.dat";
Printf("//	11 = bottom to corner of hull, outlet side	//") >> "meshPropeller.dat";
Printf("//	12 = outlet to corner of hull				//") >> "meshPropeller.dat";
Printf("//	13 = outlet to top of hull					//") >> "meshPropeller.dat";
Printf("//	14 = inlet to outlet						//") >> "meshPropeller.dat";
Printf("//////////////////////////////////////////////////") >> "meshPropeller.dat";

Printf("") >> "meshPropeller.dat";

Printf("___| Smallest cell length | Largest cell length | Expansion ratio | Length of line | Number of cells |") >> "meshPropeller.dat";
Printf(" 0 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[0], cellMax[0], r_exp[0], L_vec[0], nCells[0]) >> "meshPropeller.dat";
Printf(" 1 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[1], cellMax[1], r_exp[1], L_vec[1], nCells[1]) >> "meshPropeller.dat";
Printf(" 2 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[2], cellMax[2], r_exp[2], L_vec[2], nCells[2]) >> "meshPropeller.dat";
Printf(" 3 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[3], cellMax[3], r_exp[3], L_vec[3], nCells[3]) >> "meshPropeller.dat";
Printf(" 4 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[4], cellMax[4], r_exp[4], L_vec[4], nCells[4]) >> "meshPropeller.dat";
Printf(" 5 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[5], cellMax[5], r_exp[5], L_vec[5], nCells[5]) >> "meshPropeller.dat";
Printf(" 6 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[6], cellMax[6], r_exp[6], L_vec[6], nCells[6]) >> "meshPropeller.dat";
Printf(" 7 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[7], cellMax[7], r_exp[7], L_vec[7], nCells[7]) >> "meshPropeller.dat";
Printf(" 8 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[8], cellMax[8], r_exp[8], L_vec[8], nCells[8]) >> "meshPropeller.dat";
Printf(" 9 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[9], cellMax[9], r_exp[9], L_vec[9], nCells[9]) >> "meshPropeller.dat";
Printf("10 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[10], cellMax[10], r_exp[10], L_vec[10], nCells[10]) >> "meshPropeller.dat";
Printf("11 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[11], cellMax[11], r_exp[11], L_vec[11], nCells[11]) >> "meshPropeller.dat";
Printf("12 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[12], cellMax[12], r_exp[12], L_vec[12], nCells[12]) >> "meshPropeller.dat";
Printf("13 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[13], cellMax[13], r_exp[13], L_vec[13], nCells[13]) >> "meshPropeller.dat";
Printf("14 | %20.4f | %19.4f | %15.3f | %14.3f | %15.1f |", cellMin[14], cellMax[14], r_exp[14], L_vec[14], nCells[14]) >> "meshPropeller.dat";

Printf("") >> "meshPropeller.dat";

Printf("Total number of grid cells:") >> "meshPropeller.dat";
Printf("nCells = %12.2f", nCells_tot) >> "meshPropeller.dat";

//--------------------------------------------//
//	Write out CFD model specific data to file //
//--------------------------------------------//
//+
rho = 1025;
CFL_max = {1, 0.75, 0.5, 0.25};               // Courant-Friedrichs-Lewy
BP = 67 * 0.8 * 10^3 * 0.5;                   // [kg] Use 80% of Bollard pull
nu_s = 1.3604e-6;
Re_s = Breadth * currentVelocity / nu_s;

nu_CFD = currentVelocity*Breadth*scaleRatio / Re_s;

k_inlet = 1.5 * (currentVelocity * turbulenceIntensity)^2;
Omega_inlet = Sqrt(k_inlet) /  (0.09^(0.25)*0.07*zh);

U_s = BP / (Pi*(thrusterDiameter * 0.5)^2 * rho);
U_m = U_s * Sqrt(scaleRatio);

dt_max[0] = CFL_max[0] * fine / U_m;
dt_max[1] = CFL_max[1] * fine / U_m;
dt_max[2] = CFL_max[2] * fine / U_m;
dt_max[3] = CFL_max[3] * fine / U_m;

v_dot = U_m * Pi * (pDia * 0.5)^2;
m_dot = v_dot * rho;

// Volume and mass flow rate for U_m = 0.1 m/s
v_dot1 = 0.1 * Pi * (pDia * 0.5)^2;
m_dot1 = v_dot1 * rho;

// Scale Ratio for U_m = 1 m/s
sR = U_s^2;

Printf("") >> "meshPropeller.dat";

Printf("Adjusted kinematic viscosity for CFD purposes:") >> "meshPropeller.dat";
Printf("nu_CFD = %1.4e", nu_CFD) >> "meshPropeller.dat";

Printf("") >> "meshPropeller.dat";

Printf("Turbulent kinetic energy and Specific turbulent dissipation:") >> "meshPropeller.dat";
Printf("k and Omega:") >> "meshPropeller.dat";
Printf("k_inlet = 1.5 (U_inf * I)^2 = %6.7f", k_inlet) >> "meshPropeller.dat";
Printf("Omega_inlet = k_inlet^(0.5) / (C_mu^(0.25) * 0.07 * L) = %6.7f", Omega_inlet) >> "meshPropeller.dat";

Printf("") >> "meshPropeller.dat";

Printf("Propeller escape velocity:") >> "meshPropeller.dat";
Printf("U_s = %3.4f", U_s) >> "meshPropeller.dat";
Printf("U_m = %3.4f", U_m) >> "meshPropeller.dat";

Printf("") >> "meshPropeller.dat";

Printf("Volumetric flow rate through propeller disk:") >> "meshPropeller.dat";
Printf("volumeFlow = %6.5f [m^3/s]", v_dot) >> "meshPropeller.dat";

Printf("") >> "meshPropeller.dat";

Printf("Mass flow through propeller disk:") >> "meshPropeller.dat";
Printf("massFlow = %6.5f [kg/s]", m_dot) >> "meshPropeller.dat";

Printf("") >> "meshPropeller.dat";

Printf("Largest possible timestep:") >> "meshPropeller.dat";
Printf("C_1    = %1.2e", dt_max[0]) >> "meshPropeller.dat";
Printf("C_0.75 = %1.2e", dt_max[1]) >> "meshPropeller.dat";
Printf("C_0.5  = %1.2e", dt_max[2]) >> "meshPropeller.dat";
Printf("C_0.25 = %1.2e", dt_max[3]) >> "meshPropeller.dat";

Printf("") >> "meshPropeller.dat";

Printf("Scale ratio for U_m = 1 m/s:") >> "meshPropeller.dat";
Printf("sR = %6.5f [-]", sR) >> "meshPropeller.dat";

Printf("") >> "meshPropeller.dat";

Printf("Volume and mass flow rate for U_m = 0.1 m/s:") >> "meshPropeller.dat";
Printf("v_dot = %2.6f [m^3/s]", v_dot1) >> "meshPropeller.dat";
Printf("m_dot = %2.6f [kg/s]", m_dot1) >> "meshPropeller.dat";

//--------------------------------------------//
//	       Engage Meshing In GUI	          //
//--------------------------------------------//
//+
Mesh 3;

